unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, MMSystem, lame_enc, ExtCtrls, Buttons, ScktComp,
  ComCtrls, INIFiles;

const AudioBufferCount=8;  // Anzahl der Audio-Frames im Pufferzyklus
      MinimizedHeight=2;   // gre eines zusammengeklappten Panels

type
  TMainForm = class(TForm)
    InfoTimer: TTimer;
    TheBlackPanel: TPanel;
    LTimeActive: TLabel;
    PStartStop: TPanel;
    BStartStop: TSpeedButton;
    LCPULabel: TLabel;
    LCPU: TLabel;
    PCPU: TPanel;
    ICPU: TShape;
    Server: TServerSocket;
    PInfo: TPanel;
    LAME_DLL: TLabel;
    LAME_Engine: TLabel;
    LAME_URL: TLabel;
    PInput: TPanel;
    CDevices: TComboBox;
    CInput: TPanel;
    CClients: TPanel;
    CInfo: TPanel;
    COutput: TPanel;
    POutput: TPanel;
    PClients: TPanel;
    ConnList: TListBox;
    Spacer: TShape;
    CSampleRate: TComboBox;
    CChannels: TComboBox;
    LInputSettings: TLabel;
    CMode: TComboBox;
    CBitrate: TComboBox;
    CQuality: TComboBox;
    CVBRQuality: TTrackBar;
    COutFile: TCheckBox;
    CTimeStamp: TCheckBox;
    EOutFile: TEdit;
    BOutFile: TButton;
    COutStream: TCheckBox;
    EPort: TEdit;
    EStreamName: TEdit;
    DOutFile: TSaveDialog;
    LOutputSettings: TLabel;
    EZStreamVer: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BStartClick(Sender: TObject);
    procedure BStopClick(Sender: TObject);
    procedure InfoTimerTimer(Sender: TObject);
    procedure BStartStopClick(Sender: TObject);
    procedure ServerClientConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerClientDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerClientRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure ServerClientError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ShowHidePanel(Sender: TObject);
    procedure CModeChange(Sender: TObject);
    procedure EncodeSettingsChange(Sender: TObject);
    procedure COutFileClick(Sender: TObject);
    procedure BOutFileClick(Sender: TObject);
    procedure COutStreamClick(Sender: TObject);
  private
    { Private declarations }
    OutFD:file;                 // Ausgabedatei
    WaveIn:HWaveIn;             // Handle des verwendeten WaveIn-Devices
    LAME:HBE_STREAM;            // Handle des LAME-Encoder-Streams
    Samples:cardinal;           // Anzahl der Samples pro Puffer
    MP3Buf:pointer;             // Puffer fr MP3-Daten
    MP3BufSize:cardinal;        // Gre des MP3-Datenpuffers
    RecordStartAt:integer;      // Zeitpunkt, zu dem Aufnahme gestartet wurde
    LastTick:integer;           // letzte Aktualisierung des CPU-Meters
    PCFreq:double;              // PerformanceCounter-Frequenz
    Tixum:int64;                // Zeitbedarf des Encoders in der letzten Sekunde
    OutStream,OutFile:boolean;  // Ausgabe als Stream/Datei aktiv?
    NominalBitrate:integer;     // erwartete Bitrate (wird Clients mitgeteilt)
    // Handler fr die WaveIn-Messages
    procedure OnWaveOpen(var msg:TMessage); message MM_WIM_OPEN;
    procedure OnWaveClose(var msg:TMessage); message MM_WIM_CLOSE;
    procedure OnWaveData(var msg:TMessage); message MM_WIM_DATA;
    // selbsterklrend ...
    procedure LoadConfig;
    procedure SaveConfig;
  public
    { Public declarations }
    Recording:boolean;          // Aufnahme aktiv?
    ValidSettings:boolean;      // Aufnahmeeinstellungen gltig?
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

////////////////////////////////////////////////////////////////////////////////
// div. Konstanten, um die ItemIndexe der Einstellungs-Comboboxen auszuwerten //
////////////////////////////////////////////////////////////////////////////////
const _SAMPLERATE:array[0..8]of integer=
        (8000,11025,12000,16000,22050,24000,32000,44100,48000);
      _SAMPLERATE_MPEGLEVEL:array[0..8]of integer=(2,2,2,2,2,2,1,1,1);
const _CHANNELS_MONO=0;
      _CHANNELS_STEREO=1;
      _CHANNELS_JS=2;
const _MODE_CBR=0;
      _MODE_ABR=1;
      _MODE_VBR=2;
      _MODE_R3MIX=3;
const _BITRATE:array[0..17]of integer=
        (8,16,24,32,40,48,56,64,80,96,112,128,144,160,192,224,256,320);
      _BITRATE_MPEGLEVEL:array[0..17]of integer=
        (2,2, 2, 3, 3, 3, 3, 3, 3, 3, 3,  3,  2,  3,  1,  1,  1,  1);
const _QUALITY_LOW=0;
      _QUALITY_MEDIUM=1;
      _QUALITY_HIGH=2;
const _QUALITY:array[0..2]of integer=($FF09-$0900,$FF05-$0500,$FF02-$0200);

{$I LoadSaveConfig.inc}  // Konfiguration laden/speichern

////////////////////////////////////////////////////////////////////////////////
// Initialisierung und Deinitialisierung                                      //
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.FormCreate(Sender: TObject);
var i,max:integer;
    caps:TWaveInCaps;
    LI:BE_VERSION;
    IsMMX:string[7];
    tFreq:int64;
    MyName:string;
    FVISize:integer; FVI:array of byte;
    VerPtr:^VS_FIXEDFILEINFO;
begin
//AllocConsole;
  Recording:=false;
  SetPriorityClass(GetCurrentProcess(),HIGH_PRIORITY_CLASS);
  QueryPerformanceFrequency(tFreq); PCFreq:=tFreq;
  // Devices abfragen
  CDevices.Items.Clear;
  max:=WaveInGetNumDevs()-1;
  if max<0 then begin
    MessageDlg('Keine Aufnahmegerte gefunden!',mtError,[mbOK],0);
    halt;
  end;
  for i:=0 to max do
    if waveInGetDevCaps(i,@caps,sizeof(caps))=MMSYSERR_NOERROR then
      CDevices.Items.Add(caps.szPname);
  CDevices.ItemIndex:=0;
  // LAME-DLL abfragen
  beVersion(@LI);
  LAME_DLL.Caption:='LAME DLL '+IntToStr(LI.byDLLMajorVersion)+'.'+
    IntToStr(LI.byDLLMinorVersion)+' ('+IntToStr(LI.wYear)+'-'+
    char(48+LI.byMonth DIV 10)+char(48+LI.byMonth MOD 10)+'-'+
    char(48+LI.byDay DIV 10)+char(48+LI.byDay MOD 10)+')';
  if (LI.byMMXEnabled<>0) then IsMMX:=' MMX' else IsMMX:='';
  LAME_Engine.Caption:='LAME Engine '+IntToStr(LI.byMajorVersion)+'.'+
    IntToStr(LI.byMinorVersion)+'-'+IntToStr(LI.byBetaLevel)+'.'+
    IntToStr(LI.byAlphaLevel)+IsMMX;
  LAME_URL.Caption:='LAME URL: '+LI.zHomepage;
  // eigene Versionsnummer erfragen
  MyName:=ParamStr(0)+#0;
  FVISize:=GetFileVersionInfoSize(@MyName[1],cardinal(i));
  SetLength(FVI,FVISize);
  if (FVISize>0) AND GetFileVersionInfo(@MyName[1],0,FVISize,@FVI[0]) AND
     VerQueryValue(@FVI[0],'\',pointer(VerPtr),cardinal(i))
  then begin
    EZStreamVer.Caption:='EZStream '+
      IntToStr(VerPtr^.dwFileVersionMS SHR 16)+'.'+
      IntToStr(VerPtr^.dwFileVersionMS AND $FFFF)+'.'+
      IntToStr(VerPtr^.dwFileVersionLS SHR 16)+'.'+
      IntToStr(VerPtr^.dwFileVersionLS AND $FFFF);
    i:=VerPtr^.dwFileDateLS;
  end;
  // Alle Panels zusammenklappen
  ShowHidePanel(CInput);
  ShowHidePanel(COutput);
  ShowHidePanel(CClients);
  ShowHidePanel(CInfo);
  // diverse Voreinstellungen treffen
  CSampleRate.ItemIndex:=7;  // = 44100 Hz
  CChannels.ItemIndex:=_CHANNELS_JS;
  CMode.ItemIndex:=_MODE_CBR;
  CBitrate.ItemIndex:=11;  // = 128 kbps
  CQuality.ItemIndex:=_QUALITY_HIGH;
  // Konfiguration laden & anwenden
  LoadConfig;
  EOutFile.SelStart:=10000;
  COutFileClick(nil);
  COutStreamClick(nil);
  EncodeSettingsChange(nil);
  // Autostart
  if ParamStr(1)='-start' then BStartClick(nil);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if Recording then BStopClick(nil);
  SaveConfig;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Recording then BStopClick(nil);
end;

////////////////////////////////////////////////////////////////////////////////
// Aufnahme starten                                                           //                                                         
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.BStartClick(Sender: TObject);
var fmt:TWaveFormatEx;
    i:integer;
    Buffer:PWaveHdr;
    cfg:BE_CONFIG;
    OFN,OFNExt:string;
begin
  if not ValidSettings then begin
    BStartStop.Down:=false;
    exit;
  end;
  // SCHRITT 1: LAME initialisieren
  FillChar(cfg,sizeof(cfg),0);
  with cfg do begin
    dwConfig:=BE_CONFIG_LAME;
    dwStructVersion:=1;
    dwStructSize:=sizeof(cfg);
    dwSampleRate:=_SAMPLERATE[CSampleRate.ItemIndex];
    dwReSampleRate:=dwSampleRate;
    case CChannels.ItemIndex of
      _CHANNELS_MONO:nMode:=BE_MP3_MODE_MONO;
      _CHANNELS_STEREO:nMode:=BE_MP3_MODE_STEREO;
      _CHANNELS_JS:nMode:=BE_MP3_MODE_JSTEREO;
    end;
    nPreset:=LQP_NOPRESET;
    if _SAMPLERATE_MPEGLEVEL[CSampleRate.ItemIndex]=1 then begin
      dwMpegVersion:=MPEG1;
      dwBitrate:=32;
      dwMaxBitrate:=320;
    end else begin
      dwMpegVersion:=MPEG2;
      dwBitrate:=8;
      dwMaxBitrate:=160;
    end;
    dwPsyModel:=0;
    dwEmphasis:=0;
    bPrivate:=false;
    bCRC:=false;
    bCopyright:=false;
    bOriginal:=true;
    bWriteVBRHeader:=false;
    bEnableVBR:=true;
    bNoRes:=false;
    bStrictIso:=true;
    nQuality:=_QUALITY[CQuality.ItemIndex];
    case CMode.ItemIndex of
      _MODE_CBR:begin
                  bEnableVBR:=false;
                  dwBitrate:=_BITRATE[CBitrate.ItemIndex];
                end;
      _MODE_ABR:begin
                  nVbrMethod:=VBR_METHOD_ABR;
                  dwVbrAbr_bps:=_BITRATE[CBitrate.ItemIndex]*1000;
                end;
      _MODE_VBR:begin
                  nVbrMethod:=VBR_METHOD_DEFAULT;
                  nVbrQuality:=9-CVBRQuality.Position;
                end;
      _MODE_R3MIX:nPreset:=LQP_R3MIX;
    end;
  end;
  if beInitStream(cfg,Samples,MP3BufSize,LAME)<>BE_ERR_SUCCESSFUL then begin
    MessageDlg('Konnte LAME nicht initialisieren!',mtError,[mbOK],0);
    BStartStop.Down:=false;
    exit;
  end;
  GetMem(MP3Buf,MP3BufSize);
  // SCHRITT 2: Ausgabe initialisieren
  OutStream:=COutStream.Checked;
  OutFile:=COutFile.Checked;
  // Schritt 2a: Dateiausgabe
  if OutFile then begin
    OFN:=EOutFile.Text;
    OFNExt:=ExtractFileExt(OFN);
    if CTimeStamp.Checked then
      OFN:=copy(OFN,1,length(OFN)-length(OFNExt))+
           FormatDateTime('-yyyy-mm-dd-hh-nn',Now)+OFNExt;
    AssignFile(OutFD,OFN);
    {$I-} Rewrite(OutFD,1); {$I+}
    if IOResult<>0 then begin
      MessageDlg('Konnte Ausgabedatei nicht ffnen!',mtError,[mbOK],0);
      BStartStop.Down:=false;
      exit;
    end;
  end;
  // Schritt 2b: HTTP-Stream-Ausgabe
  if OutStream then begin
    try
      Server.Active:=true;
    except
      MessageDlg('Konnte HTTP-Socket nicht initialisieren!',mtError,[mbOK],0);
      CloseFile(OutFD);
      BStartStop.Down:=false;
      exit;
    end;
    NominalBitrate:=_BITRATE[CBitrate.ItemIndex];
  end;
  // SCHRITT 3: Wave-Capturing initialisieren
  // Schritt 3a: gewnschten Format-Tag konstruieren
  with fmt do begin
    wFormatTag:=WAVE_FORMAT_PCM;
    if CChannels.ItemIndex=_CHANNELS_MONO then nChannels:=1 else nChannels:=2;
    nSamplesPerSec:=_SAMPLERATE[CSampleRate.ItemIndex];
    nAvgBytesPerSec:=_SAMPLERATE[CSampleRate.ItemIndex]*nChannels*2;
    nBlockAlign:=nChannels*2;
    wBitsPerSample:=16;
    cbSize:=0;
  end;
  // Schritt 3b: WaveIn-Device ffnen
  if waveInOpen(@WaveIn,CDevices.ItemIndex,@fmt,Handle,0,CALLBACK_WINDOW)<>
    MMSYSERR_NOERROR then begin
      MessageDlg('Konnte WaveIn nicht ffnen!',mtError,[mbOK],0);
      CloseFile(OutFD);
      BStartStop.Down:=false;
      exit;
    end;
  // Schritt 3c: Audio-Puffer anlegen
  for i:=1 to AudioBufferCount do begin
    New(Buffer);
    // Schritt 3c-1: Puffer-Header ausfllen
    with Buffer^ do begin
      dwBufferLength:=Samples SHL 1;
      GetMem(lpData,dwBufferLength);
      dwBytesRecorded:=0;
      dwUser:=0;
      dwFlags:=0;
      dwLoops:=0;
    end;
    // Schritt 3c-2: Header vorbereiten
    if waveInPrepareHeader(WaveIn,Buffer,sizeof(TWaveHdr))<>
      MMSYSERR_NOERROR then begin
        MessageDlg('Konnte Puffer nicht vorbereiten!',mtError,[mbOK],0);
        CloseFile(OutFD);
        BStartStop.Down:=false;
        exit;
      end;
    // Schritt 3c-3: Header in den Zyklus schicken
    if waveInAddBuffer(WaveIn,Buffer,sizeof(TWaveHdr))<>
      MMSYSERR_NOERROR then begin
        MessageDlg('Konnte Puffer nicht hinzufgen!',mtError,[mbOK],0);
        CloseFile(OutFD);
        BStartStop.Down:=false;
        exit;
      end;
  end;
  // Schritt 3d: Aufnahme starten
  if waveInStart(WaveIn)<>MMSYSERR_NOERROR then begin
    MessageDlg('Konnte Aufnahme nicht starten!',mtError,[mbOK],0);
    CloseFile(OutFD);
    BStartStop.Down:=false;
    exit;
  end;
  Recording:=true;
end;

////////////////////////////////////////////////////////////////////////////////
// Aufname beenden; sonstige Start-/Stop-Handler                              //
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.BStopClick(Sender: TObject);
var MP3Size:cardinal;
    Dummy:integer;
begin
  Recording:=false;
  waveInStop(WaveIn);
  waveInReset(WaveIn);
  waveInClose(WaveIn);
  if (beDeinitStream(LAME,MP3Buf,MP3Size)=BE_ERR_SUCCESSFUL) AND OutFile
    then BlockWrite(OutFD,MP3Buf^,MP3Size,Dummy);
  beCloseStream(LAME);
  Server.Active:=false;
  if OutFile then begin
    {$I-} CloseFile(OutFD); {$I+}
  end;
end;

procedure TMainForm.BStartStopClick(Sender: TObject);
begin
  if Recording then BStopClick(nil) else BStartClick(nil);
end;

procedure TMainForm.OnWaveOpen(var msg:TMessage);
begin
  BStartStop.Down:=true;
  RecordStartAt:=GetTickCount();
  LastTick:=-1;
  CClients.Caption:='  Clients (0)';
  InfoTimerTimer(nil);
  InfoTimer.Enabled:=true;
end;

procedure TMainForm.OnWaveClose(var msg:TMessage);
begin
  InfoTimer.Enabled:=false;
  BStartStop.Down:=false;
  LCPU.Caption:='0%';
  ICPU.Width:=0;
  ConnList.Items.Clear;
  CClients.Caption:='  Clients';
end;

////////////////////////////////////////////////////////////////////////////////
// Audio-Frame handlen                                                        //  
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.OnWaveData(var msg:TMessage);
var Buffer:PWaveHdr;
    MP3Size:cardinal;
    tStart,tEnd:int64;
    Client:integer;
begin
  QueryPerformanceCounter(tStart);
  Buffer:=PWaveHdr(msg.LParam);
  // Phase 1: Puffer encoden, abspeichern und an die Clients schicken
  with Buffer^ do begin
    if Recording AND (dwBytesRecorded SHR 1=Samples) then
      if beEncodeChunk(LAME,Samples,@(lpData^),MP3Buf,MP3Size)=BE_ERR_SUCCESSFUL
        then begin
          if OutFile then
            BlockWrite(OutFD,MP3Buf^,MP3Size,Client);
          if OutStream then
            for Client:=0 to (Server.Socket.ActiveConnections-1) do
              with Server.Socket.Connections[Client] do
                if Connected AND (Data<>nil) then begin
                  try
                    SendBuf(MP3Buf^,MP3Size);
                  except
                    // Fehler ignorieren
                  end;
                end;
        end;
    waveInUnprepareHeader(WaveIn,PWaveHdr(msg.LParam),sizeof(TWaveHdr));
  end;
  // Phase 2: Puffer fr neue Runde vorbereiten
  if Recording then begin
    with Buffer^ do begin
      dwBufferLength:=Samples SHL 1;
      dwBytesRecorded:=0;
      dwUser:=0;
      dwFlags:=0;
      dwLoops:=0;
    end;
    waveInPrepareHeader(WaveIn,Buffer,sizeof(TWaveHdr));
    WaveInAddBuffer(WaveIn,Buffer,sizeof(TWaveHdr));
  end else
    Dispose(PWaveHdr(msg.LParam));
  QueryPerformanceCounter(tEnd);
  inc(Tixum,tEnd-tStart);
end;

////////////////////////////////////////////////////////////////////////////////
// Timerevent zur Berechnung der CPU-Last des Encoders                        //
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.InfoTimerTimer(Sender: TObject);
var H,M,S:integer;
    Tick:integer;
    CPU:integer;
begin
  Tick:=GetTickCount();
  S:=(Tick-RecordStartAt)DIV 1000;
  H:=S DIV 3600;
  M:=(S DIV 60) MOD 60;
  S:=S MOD 60;
  LTimeActive.Caption:=IntToStr(H)+':'+char(48+M DIV 10)+char(48+M MOD 10)+':'+
    char(48+S DIV 10)+char(48+S MOD 10);
  if LastTick>=0 then begin
    CPU:=round((Tixum/PCFreq)/(0.001*(Tick-LastTick))*100.0);
    LCPU.Caption:=IntToStr(CPU)+'%';
    ICPU.Width:=CPU;
  end;
  LastTick:=Tick;
  Tixum:=0;
end;

////////////////////////////////////////////////////////////////////////////////
// Client-Connections handlen                                                 //
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.ServerClientConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  Socket.Data:=nil;  // nil=Pointer(0), zum Glck
  ConnList.Items.Add(Socket.RemoteHost);
  CClients.Caption:='  Clients ('+IntToStr(Server.Socket.ActiveConnections)+')';
end;

procedure TMainForm.ServerClientDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
var Index:integer;
begin
  Index:=ConnList.Items.IndexOf(Socket.RemoteHost);
  if Index>=0 then ConnList.Items.Delete(Index);
  CClients.Caption:='  Clients ('+IntToStr(Server.Socket.ActiveConnections-1)+')';
end;

procedure TMainForm.ServerClientRead(Sender: TObject;
  Socket: TCustomWinSocket);
var s:string;
begin
  s:=Socket.ReceiveText;
  if Socket.Data<>nil then exit;
  if pos(^M^J^M^J,s)<0 then exit;
  Socket.SendText(
    'ICY 200 OK'^M^J+
    'icy-notice1:This stream is served by EZStream'^M^J+
    'icy-notice2:Note that this server max crash like hell :)'^M^J+
    'icy-name:'+EStreamName.Text+^M^J+
    'icy-br:'+IntToStr(NominalBitrate)+^M^J^M^J);
  Socket.Data:=Pointer(true);
end;

procedure TMainForm.ServerClientError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  ErrorCode:=0;
  Socket.Close;
end;

////////////////////////////////////////////////////////////////////////////////
// All dieses langweilige, zeitraubende Benutzeroberflchengeraffel           //   
////////////////////////////////////////////////////////////////////////////////
procedure TMainForm.ShowHidePanel(Sender: TObject);
var Control:TControl;
begin
       if Sender=CInput   then Control:=PInput
  else if Sender=COutput  then Control:=POutput
  else if Sender=CClients then Control:=PClients
  else if Sender=CInfo    then Control:=PInfo
  else exit;
  if Control.Height=MinimizedHeight then begin
    Control.Height:=Control.Tag;
    (Sender as TPanel).BevelOuter:=bvLowered;
  end else begin
    Control.Tag:=Control.Height;
    Control.Height:=MinimizedHeight;
    (Sender as TPanel).BevelOuter:=bvRaised;
  end;
end;

procedure TMainForm.CModeChange(Sender: TObject);
var M:integer;
begin
  M:=CMode.ItemIndex;
  CVBRQuality.Visible:=(M=_MODE_VBR);
  CBitrate.Visible:=(M<_MODE_VBR);
  EncodeSettingsChange(Sender);
end;

procedure TMainForm.EncodeSettingsChange(Sender: TObject);
var s:string;
    VSI,VSO:boolean;
    TOS,TOF:boolean;
begin
  if Recording then exit;
  VSI:=true; VSO:=true;
  s:=IntToStr(_SAMPLERATE[CSampleRate.ItemIndex] DIV 1000)+'kHz ';
  case CChannels.ItemIndex of
    _CHANNELS_MONO:s:=s+'Mono, ';
    _CHANNELS_STEREO:s:=s+'Stereo, ';
    _CHANNELS_JS:s:=s+'JS, ';
  end;
  case CMode.ItemIndex of
    _MODE_CBR:if (_SAMPLERATE_MPEGLEVEL[CSampleRate.ItemIndex] AND
                  _BITRATE_MPEGLEVEL[CBitrate.ItemIndex])=0
              then VSI:=false
              else s:=s+IntToStr(_BITRATE[CBitrate.ItemIndex])+'kbps CBR';
    _MODE_ABR:s:=s+IntToStr(_BITRATE[CBitrate.ItemIndex])+'kbps ABR';
    _MODE_VBR:s:=s+'VBR q='+IntToStr(CVBRQuality.Position);
    _MODE_R3MIX:s:=s+'r3mix';
  end;
  case CQuality.ItemIndex of
    _QUALITY_LOW:s:=s+', LQ';
    _QUALITY_MEDIUM:s:=s+', MQ';
    _QUALITY_HIGH:s:=s+', HQ';
  end;
  if VSI then begin
    LInputSettings.Caption:=s;
    LInputSettings.Font.Color:=clLime;
  end else begin
    LInputSettings.Caption:='ungltiges Format';
    LInputSettings.Font.Color:=clRed;
  end;
  TOS:=COutStream.Checked;
  TOF:=COutFile.Checked;
  if not Server.Active then begin
    try
      Server.Port:=StrToInt(EPort.Text);
    except
      VSO:=false;
    end;
  end;
  if TOS then s:='Port '+IntToStr(Server.Port) else s:='';
  if (TOS AND TOF) then s:=s+' + ';
  if TOF then s:=s+ExtractFileName(EOutFile.Text);
  if not(TOS OR TOF) then s:='keine Ausgabe';
  if VSO then begin
    LOutputSettings.Caption:=s;
    if TOS OR OutFile
      then LOutputSettings.Font.Color:=clLime
      else LOutputSettings.Font.Color:=clRed;
  end else begin
    LOutputSettings.Caption:='ungltige Ausgabeeinstellungen';
    LOutputSettings.Font.Color:=clRed;
  end;
  ValidSettings:=VSI AND VSO;
end;

procedure TMainForm.COutFileClick(Sender: TObject);
var z:boolean;
begin
  z:=COutFile.Checked;
  CTimeStamp.Enabled:=z;
  EOutFile.Enabled:=z;
  EncodeSettingsChange(nil);
end;

procedure TMainForm.BOutFileClick(Sender: TObject);
begin
  if not DOutFile.Execute then exit;
  EOutFile.Text:=DOutFile.FileName;
  EOutFile.SelStart:=10000;
  COutFile.Checked:=true;
  COutFileClick(nil);
end;

procedure TMainForm.COutStreamClick(Sender: TObject);
var z:boolean;
begin
  z:=COutStream.Checked;
  EPort.Enabled:=z;
  EStreamName.Enabled:=z;
  EncodeSettingsChange(nil);
end;

end.
