unit lame_enc;
// lame_enc: yet another LAME Encoder DLL header translation to Delphi
interface

// encoding formats
const BE_CONFIG_MP3  = 0;
const BE_CONFIG_LAME = 256;

// type definitions
type HBE_STREAM  = cardinal;
type PHBE_STREAM = ^HBE_STREAM;
type BE_ERR      = cardinal;

// error codes
const BE_ERR_SUCCESSFUL                = $00000000;
const BE_ERR_INVALID_FORMAT            = $00000001;
const BE_ERR_INVALID_FORMAT_PARAMETERS = $00000002;
const BE_ERR_NO_MORE_HANDLES           = $00000003;
const BE_ERR_INVALID_HANDLE            = $00000004;
const BE_ERR_BUFFER_TOO_SMALL          = $00000005;

// other constants
const BE_MAX_HOMEPAGE = 128;

// format specific variables
const BE_MP3_MODE_STEREO      = 0;
const BE_MP3_MODE_JSTEREO     = 1;
const BE_MP3_MODE_DUALCHANNEL = 2;
const BE_MP3_MODE_MONO        = 3;

const MPEG1 = 1;
const MPEG2 = 0;

type VBRMETHOD = integer;
const VBR_METHOD_NONE    = -1;
const VBR_METHOD_DEFAULT =  0;
const VBR_METHOD_OLD     =  1;
const VBR_METHOD_NEW     =  2;
const VBR_METHOD_MTRH    =  3;
const VBR_METHOD_ABR     =  4;

type LAME_QUALITY_PRESET = integer;
const LQP_NOPRESET         =-1;
// QUALITY PRESETS
const LQP_NORMAL_QUALITY   = 0;
const LQP_LOW_QUALITY      = 1;
const LQP_HIGH_QUALITY     = 2;
const LQP_VOICE_QUALITY    = 3;
const LQP_R3MIX            = 4;
const LQP_VERYHIGH_QUALITY = 5;
const LQP_STANDARD         = 6;
const LQP_FAST_STANDARD    = 7;
const LQP_EXTREME          = 8;
const LQP_FAST_EXTREME     = 9;
const LQP_INSANE           = 10;
const LQP_ABR              = 11;
const LQP_CBR              = 12;
const LQP_MEDIUM           = 13;
const LQP_FAST_MEDIUM      = 14;
// NEW PRESET VALUES
const LQP_PHONE            =1000;
const LQP_SW               =2000;
const LQP_AM               =3000;
const LQP_FM               =4000;
const LQP_VOICE            =5000;
const LQP_RADIO            =6000;
const LQP_TAPE             =7000;
const LQP_HIFI             =8000;
const LQP_CD               =9000;
const LQP_STUDIO           =10000;

type PBE_CONFIG = ^BE_CONFIG;
     BE_CONFIG = packed record  // LHV1 only!!!
                   dwConfig:cardinal; // BE_CONFIG_XXXXX
                                      // Currently only BE_CONFIG_MP3 is supported
                   dwStructVersion:cardinal;
                   dwStructSize:cardinal;
                   // BASIC ENCODER SETTINGS
                   dwSampleRate:cardinal;     // SAMPLERATE OF INPUT FILE
                   dwReSampleRate:cardinal;   // DOWNSAMPLERATE, 0=ENCODER DECIDES
                   nMode:integer;             // BE_MP3_MODE_STEREO, BE_MP3_MODE_DUALCHANNEL, BE_MP3_MODE_MONO
                   dwBitrate:cardinal;        // CBR bitrate, VBR min bitrate
                   dwMaxBitrate:cardinal;     // CBR ignored, VBR Max bitrate
                   nPreset:integer;           // Quality preset, use one of the settings of the LAME_QUALITY_PRESET enum
                   dwMpegVersion:cardinal;    // FUTURE USE, MPEG-1 OR MPEG-2
                   dwPsyModel:cardinal;       // FUTURE USE, SET TO 0
                   dwEmphasis:cardinal;       // FUTURE USE, SET TO 0
                   // BIT STREAM SETTINGS
                   bPrivate:LongBool;         // Set Private Bit (TRUE/FALSE)
                   bCRC:LongBool;             // Insert CRC (TRUE/FALSE)
                   bCopyright:LongBool;       // Set Copyright Bit (TRUE/FALSE)
                   bOriginal:LongBool;        // Set Original Bit (TRUE/FALSE)
                   // VBR STUFF
                   bWriteVBRHeader:LongBool;  // WRITE XING VBR HEADER (TRUE/FALSE)
                   bEnableVBR:LongBool;	      // USE VBR ENCODING (TRUE/FALSE)
                   nVBRQuality:integer;       // VBR QUALITY 0..9
                   dwVbrAbr_bps:cardinal;     // Use ABR in stead of nVBRQuality
                   nVbrMethod:VBRMETHOD;
                   bNoRes:LongBool;           // Disable Bit resorvoir (TRUE/FALSE)
                   // MISC SETTINGS
                   bStrictIso:LongBool;       // Use strict ISO encoding rules (TRUE/FALSE)
                   nQuality:word;             // Quality Setting, HIGH BYTE should be NOT LOW byte, otherwhise quality=5
                   // FUTURE USE, SET TO 0, align strucutre to 331 bytes
                   btReserved:array[1..255-4*sizeof(cardinal)-sizeof(word)]of byte;
                 end;

type PBE_VERSION = ^BE_VERSION;
     BE_VERSION = packed record
                    // BladeEnc DLL Version number
                    byDLLMajorVersion:byte;
                    byDLLMinorVersion:byte;
                    // BladeEnc Engine Version Number
                    byMajorVersion:byte;
                    byMinorVersion:byte;
                    // DLL Release date
                    byDay:byte;
                    byMonth:byte;
                    wYear:word;
                    // BladeEnc Homepage URL
                    zHomepage:array[0..BE_MAX_HOMEPAGE]of char;
                    byAlphaLevel:byte;
                    byBetaLevel:byte;
                    byMMXEnabled:byte;
                    btReserved:array[1..125]of byte;
                  end;

type PSHORT=^smallint;
     PBYTE=^byte;

function beInitStream(var pbeConfig:BE_CONFIG;
                      var dwSamples:cardinal;
                      var dwBufferSize:cardinal;
                      var phbeStream:HBE_STREAM)
                     :BE_ERR; cdecl; external 'lame_enc.dll';

function beEncodeChunk(hbeStream:HBE_STREAM;
                       nSamples:cardinal;
                       pSamples:PSHORT;
                       pOutput:PBYTE;
                       var pdwOutput:cardinal)
                      :BE_ERR; cdecl; external 'lame_enc.dll';

function beDeinitStream(hbeStream:HBE_STREAM;
                        pOutput:PBYTE;
                        var pdwOutput:cardinal)
                       :BE_ERR; cdecl; external 'lame_enc.dll';

function beCloseStream(hbeStream:HBE_STREAM):BE_ERR;
                       cdecl; external 'lame_enc.dll';

procedure beVersion(pbeVersion:PBE_VERSION); cdecl; external 'lame_enc.dll';

function beWriteVBRHeader(lpszFileName:PChar):BE_ERR;
                          cdecl; external 'lame_enc.dll';

function beFlushNoGap(hbeStream:HBE_STREAM;
                      pOutput:PBYTE;
                      var pdwOutput:cardinal)
                     :BE_ERR; cdecl; external 'lame_enc.dll';

function beWriteInfoTag(hbeStream:HBE_STREAM;
                        lpszFileName:PChar)
                       :BE_ERR; cdecl; external 'lame_enc.dll';

implementation

end.
